@@ -14,74 +14,85 @@ use warnings;
 
 our @ISA = qw( Parse::Nibbler );
 
-use Parse::Nibbler;
 use Data::Dumper;
 
-our $VERSION = '1.04';
+
+use Parse::Nibbler;
+
+our $VERSION = '1.05';
+
+
+use constant list_of_rules_in_progress => 0;
+use constant line_number=> 1;
+use constant current_line => 2;
+use constant handle => 3;
+use constant lexical_boneyard => 4;
+use constant filename => 5;
 
 
 #############################################################################
 # Lexer
 #############################################################################
-sub Lexer
+sub Lexer_old
 #############################################################################
 {
   my $p = $_[0];
 
   while(1)
     {
-      my $line = $p->{line_number};
-      my $col = pos($p->{current_line});
+      my $line = $p->[line_number];
+      my $col = pos($p->[current_line]);
 
       # if at end of line
       if( 
-	 ( length($p->{current_line}) == 0 )
+	 ( length($p->[current_line]) == 0 )
 	 or
-	 ( length($p->{current_line}) == pos($p->{current_line}) )
+	 ( length($p->[current_line]) == pos($p->[current_line]) )
 	 )
 	{
-	  $p->{line_number} ++;
-	  my $fh = $p->{handle};
-	  $p->{current_line} = <$fh>;
+	  $p->[line_number] ++;
+	  # print "line ". $p->[line_number]."\n";
+	  my $fh = $p->[handle];
+	  $p->[current_line] = <$fh>;
 
-	  unless(defined($p->{current_line}))
+	  unless(defined($p->[current_line]))
 	    {
 	      return bless [ '!EOF!', '!EOF!', $line, $col ], 'Lexical';
 	    }
 
-	  chomp($p->{current_line});
-	  pos($p->{current_line}) = 0;
+	  chomp($p->[current_line]);
+	  pos($p->[current_line]) = 0;
 	  redo;
 	}
 
       # delete any leading whitespace and check it again
-      if($p->{current_line} =~ /\G\s+/gc)
+      if($p->[current_line] =~ /\G\s+/gc)
 	{
 	  redo;
 	}
 
       # look for comment to end of line
-      if($p->{current_line} =~ /\G\/\/.*/gc)
+      if($p->[current_line] =~ /\G\/\/.*/gc)
 	{
 	  redo;
 	}
 
       # look for identifier (hierarchical)
       my $identifier = '';
-      if( $p->{current_line} =~ /\G\$/gc )
+      if( $p->[current_line] =~ /\G\$/gc )
 	{
 	  $identifier = "\$";   # system task name
 	}
 
       while(1)
 	{
-	  if ( $p->{current_line} =~ /\G([a-zA-Z_][a-zA-Z0-9_\$]*)/gc )
+	  if ( $p->[current_line] =~ /\G([a-zA-Z_][a-zA-Z0-9_\$]*)/gc )
 	    {$identifier .= $1;}
 
-	  elsif($p->{current_line} =~ /\G(\\[^\s]+)\s/gc) 
+	  elsif($p->[current_line] =~ /\G(\\[^\s]+)\s/gc) 
 	    {
 	      $identifier .= $1;
-	      pos($p->{current_line}) = pos($p->{current_line}) - 1; 
+	      pos($p->[current_line]) = pos($p->[current_line]) - 1; 
 	    }
 
 	  # no match and no accumulated identifier string
@@ -90,7 +101,7 @@ sub Lexer
 
 	  $p->report_error("bad identifier") if ($identifier =~ /\.$/);
 
-	  if ($p->{current_line} =~ /\G\./gc)
+	  if ($p->[current_line] =~ /\G\./gc)
 	    {
 	      $identifier .= '.';
 	      redo;
@@ -132,12 +143,12 @@ sub Lexer
       # will ignore optional sign at beginning of number for now.
       # will hope that parser can handle it in rules.
 
-      if ($p->{current_line} =~ /\G([0-9][0-9_]*)/gc)
+      if ($p->[current_line] =~ /\G([0-9][0-9_]*)/gc)
 	{
 	  $number .= $1;
 	}
 
-      if ( $p->{current_line} =~ /\G(\'[dDoObBhH])/gc )
+      if ( $p->[current_line] =~ /\G(\'[dDoObBhH])/gc )
 	{
 	  $number .= $1;
 	  $number = lc($number);
@@ -150,7 +161,7 @@ sub Lexer
 
 	  if($number=~/d$/)
 	    {
-	      $p->{current_line} =~ /\G([0-9][0-9_]*)/gc;
+	      $p->[current_line] =~ /\G([0-9][0-9_]*)/gc;
 	      unless(defined($1))
 		{
 		  $p->report_error("Lex error: invalid decimal number") 
@@ -159,7 +170,7 @@ sub Lexer
 	    }
 	  elsif($number=~/o$/)
 	    {
-	      $p->{current_line} =~ /\G([xXzZ0-7][xXzZ0-7_]*)/gc;
+	      $p->[current_line] =~ /\G([xXzZ0-7][xXzZ0-7_]*)/gc;
 	      unless(defined($1))
 		{
 		  $p->report_error("Lex error: invalid octal number") 
@@ -168,7 +179,7 @@ sub Lexer
 	    }
 	  elsif($number=~/b$/)
 	    {
-	      $p->{current_line} =~ /\G([xXzZ01][xXzZ01_]*)/gc;
+	      $p->[current_line] =~ /\G([xXzZ01][xXzZ01_]*)/gc;
 	      unless(defined($1))
 		{
 		  $p->report_error("Lex error: invalid binary number") 
@@ -177,7 +188,7 @@ sub Lexer
 
 	  elsif($number=~/h$/)
 	    {
-	      $p->{current_line} =~ /\G([xXzZ0-9a-fA-F][xXzZ0-9a-fA-F_]*)/gc;
+	      $p->[current_line] =~ /\G([xXzZ0-9a-fA-F][xXzZ0-9a-fA-F_]*)/gc;
 	      unless(defined($1))
 		{
 		  $p->report_error("Lex error: invalid hexadecimal number") 
@@ -195,12 +206,12 @@ sub Lexer
 	  return bless ['Number', $number, $line, $col ], 'Lexical';
 	}
 
-      if( $p->{current_line} =~ /\G(\.[0-9][0-9_]*)/gc )
+      if( $p->[current_line] =~ /\G(\.[0-9][0-9_]*)/gc )
 	{
 	  $number .= $1;
 	}
 
-      if( $p->{current_line} =~ /\G(\s*[eE]\s*[+-]*\s*[0-9][0-9_]*)/gc )
+      if( $p->[current_line] =~ /\G(\s*[eE]\s*[+-]*\s*[0-9][0-9_]*)/gc )
 	{
 	  $number .= $1;
 	}
@@ -214,7 +225,7 @@ sub Lexer
 	}
 
       # else get a single character and return it.
-      $p->{current_line} =~ /\G(.)/gc;
+      $p->[current_line] =~ /\G(.)/gc;
       return bless [$1, $1, $line, $col ], 'Lexical';
 
     }
@@ -224,6 +235,215 @@ sub Lexer
 
 
 
+
+#############################################################################
+# Lexer
+#############################################################################
+sub Lexer
+#############################################################################
+{
+  my $p = $_[0];
+
+  while(1)
+    {
+      my $line = $p->[line_number];
+      my $col = pos($p->[current_line]);
+
+      # if at end of line
+      if( 
+	 ( length($p->[current_line]) == 0 )
+	 or
+	 ( length($p->[current_line]) == pos($p->[current_line]) )
+	 )
+	{
+	  $p->[line_number] ++;
+	  # print "line ". $p->[line_number]."\n";
+	  my $fh = $p->[handle];
+	  $p->[current_line] = <$fh>;
+
+	  unless(defined($p->[current_line]))
+	    {
+	      return bless [ '!EOF!', '!EOF!', $line, $col ], 'Lexical';
+	    }
+
+	  chomp($p->[current_line]);
+	  pos($p->[current_line]) = 0;
+	  redo;
+	}
+
+      # delete any leading whitespace and check it again
+      if($p->[current_line] =~ /\G\s+/gc)
+	{
+	  redo;
+	}
+
+      # look for comment to end of line
+      if($p->[current_line] =~ /\G\/\/.*/gc)
+	{
+	  redo;
+	}
+
+      # look for identifier (hierarchical)
+      my $identifier = '';
+      if( $p->[current_line] =~ /\G\$/gc )
+	{
+	  $identifier = "\$";   # system task name
+	}
+
+      while(1)
+	{
+	  if ( $p->[current_line] =~ /\G([a-zA-Z_][a-zA-Z0-9_\$]*)/gc )
+	    {$identifier .= $1;}
+
+	  elsif($p->[current_line] =~ /\G(\\[^\s]+)\s/gc) 
+	    {
+	      $identifier .= $1;
+	      pos($p->[current_line]) = pos($p->[current_line]) - 1; 
+	    }
+
+	  # no match and no accumulated identifier string
+	  elsif (length($identifier) == 0)
+	    { last; } 
+
+	  $p->report_error("bad identifier") if ($identifier =~ /\.$/);
+
+	  if ($p->[current_line] =~ /\G\./gc)
+	    {
+	      $identifier .= '.';
+	      redo;
+	    }
+
+	  if ($identifier =~ /\A\$/)
+	    {
+	      if(length($identifier) == 1)
+		{
+		  return bless  ['$', '$', $line, $col ], 'Lexical';
+		}
+	      else
+		{
+		  return bless 
+		    ['system_task_name', $identifier, $line, $col ], 'Lexical';
+		}
+
+	    }
+
+	  else
+	    {
+	      return bless ['Identifier', $identifier, $line, $col ], 'Lexical';
+	    }
+
+	}
+
+      # look for a 'Number' in Verilog style of number
+      # [+-] unsigned_number 
+      # [+-] [unsigned_number] 'd unsigned_number
+      # [+-] [unsigned_number] 'o octal_number
+      # [+-] [unsigned_number] 'b binary_number
+      # [+-] [unsigned_number] 'h hex_number
+      # [+-] unsigned_number . unsigned_number
+      # [+-] unsigned_number [ . unsigned_number ] e [+-] unsigned_number
+
+      # first character must be a number or a '
+      my $number = '';
+
+      # will ignore optional sign at beginning of number for now.
+      # will hope that parser can handle it in rules.
+
+      if ($p->[current_line] =~ /\G([0-9][0-9_]*)/gc)
+	{
+	  $number .= $1;
+	}
+
+      if ( $p->[current_line] =~ /\G(\'[dDoObBhH])/gc )
+	{
+	  $number .= $1;
+	  $number = lc($number);
+
+	  # if no number was given to indicate size, default is 32
+	  unless($number =~ /\A[0-9]/)
+	    {
+	      $number = '32' . $number;
+	    }
+
+	  if($number=~/d$/)
+	    {
+	      $p->[current_line] =~ /\G([0-9][0-9_]*)/gc;
+	      unless(defined($1))
+		{
+		  $p->report_error("Lex error: invalid decimal number") 
+		}
+	      $number .= $1;
+	    }
+	  elsif($number=~/o$/)
+	    {
+	      $p->[current_line] =~ /\G([xXzZ0-7][xXzZ0-7_]*)/gc;
+	      unless(defined($1))
+		{
+		  $p->report_error("Lex error: invalid octal number") 
+		}
+	      $number .= $1;
+	    }
+	  elsif($number=~/b$/)
+	    {
+	      $p->[current_line] =~ /\G([xXzZ01][xXzZ01_]*)/gc;
+	      unless(defined($1))
+		{
+		  $p->report_error("Lex error: invalid binary number") 
+		}	      $number .= $1;
+	    }
+
+	  elsif($number=~/h$/)
+	    {
+	      $p->[current_line] =~ /\G([xXzZ0-9a-fA-F][xXzZ0-9a-fA-F_]*)/gc;
+	      unless(defined($1))
+		{
+		  $p->report_error("Lex error: invalid hexadecimal number") 
+		}	      $number .= $1;
+	    }
+	  
+	  if($number =~ /_$/)
+	    {
+	      report_error("Lex: number ended with '_'");
+	    }
+
+	  $number = lc($number);
+	  $number =~ s/_//g;
+	  $number =~ s/\s//g;
+	  return bless ['Number', $number, $line, $col ], 'Lexical';
+	}
+      else  # not a based number ('h), is it a float with exponent
+	{
+	  if( $p->[current_line] =~ /\G(\.[0-9][0-9_]*)/gc )
+	    {
+	      $number .= $1;
+	    }
+
+	  if( $p->[current_line] =~ /\G(\s*[eE]\s*[+-]*\s*[0-9][0-9_]*)/gc )
+	    {
+	      $number .= $1;
+	    }
+
+	  if(length($number)>0)
+	    {
+	      $number = lc($number);
+	      $number =~ s/\s//g;
+	      $number =~ s/_//g;
+	      return bless ['Number', $number, $line, $col ], 'Lexical';
+	    }
+	  }
+
+      # else get a single character and return it.
+      $p->[current_line] =~ /\G(.)/gc;
+      return bless [$1, $1, $line, $col ], 'Lexical';
+
+    }
+}
+
+
+
+
+
+
 ###############################################################################
 Register 
 ( 'Number', sub 
@@ -256,7 +476,7 @@ Register
 ( 'SourceText', sub 
 ###############################################################################
   {
-    $_[0]->Description('{*}');
+    $_[0]->DescriptionArgs('{*}');
   }
 );
 
@@ -277,9 +497,9 @@ Register
     my $p = $_[0];
     $p->ValueIs('module');
     $p->TypeIs('Identifier');
-    $p->ListOfPorts('{?}');
+    $p->ListOfPortsArgs('{?}');
     $p->ValueIs(';');
-    $p->ModuleItem('{*}');
+    $p->ModuleItemArgs('{*}');
     $p->ValueIs('endmodule');
   }
 );
@@ -311,7 +531,7 @@ Register
 ( 'NamedPortExpressionList', sub 
 ###############################################################################
   {
-    $_[0]->NamedPortExpression('{+}/,/');
+    $_[0]->NamedPortExpressionArgs('{+}/,/');
   }
 );
 
@@ -320,7 +540,7 @@ Register
 ( 'AnonPortExpressionList', sub 
 ###############################################################################
   {
-    $_[0]->AnonPortExpression('{+}/,/');
+    $_[0]->AnonPortExpressionArgs('{+}/,/');
   }
 );
 
@@ -334,7 +554,7 @@ Register
     $p->ValueIs('.');
     $p->TypeIs('Identifier');
     $p->ValueIs('(');
-    $p->AnonPortExpression('{?}');
+    $p->AnonPortExpressionArgs('{?}');
     $p->ValueIs(')');
   }
 );
@@ -346,7 +566,7 @@ Register
 ###############################################################################
   {
     $_[0]->AlternateRules
-      ( 
+      (
        'PortReference', 'ConcatenatedPortReference'
       );
   }
@@ -358,7 +578,7 @@ Register
 ###############################################################################
   {
     $_[0]->AlternateRules
-      ( 
+      (
        'IdentifierWithPossibleBitSpecifier', 'Number'
       );
   }
@@ -372,7 +592,7 @@ Register
   {
     my $p = $_[0];
     $p->TypeIs('Identifier');
-    $p->BitSpecifier('{?}');
+    $p->BitSpecifierArgs('{?}');
   }
 );
 
@@ -386,7 +606,7 @@ Register
     my $p = $_[0];
     $p->ValueIs('[');
     $p->TypeIs('Number');
-    $p->ColonNumber('{?}');
+    $p->ColonNumberArgs('{?}');
     $p->ValueIs(']');
   }
 );
@@ -409,7 +629,7 @@ Register
   {
     my $p = $_[0];
     $p->ValueIs('{');
-    $p->PortReference('{+}/,/');
+    $p->PortReferenceArgs('{+}/,/');
     $p->ValueIs('}');
   }
 );
@@ -435,8 +655,8 @@ Register
   {
     my $p = $_[0];
     $p->AlternateValues('input', 'output', 'inout');
-    $p->Range('{?}');
-    $p->PortIdentifier('{+}');
+    $p->RangeArgs('{?}');
+    $p->PortIdentifierArgs('{+}');
     $p->ValueIs(';');
   }
 );
@@ -473,8 +693,8 @@ Register
   {
     my $p = $_[0];
     $p->TypeIs('Identifier');
-    $p->ParameterValueAssignment('{?}');
-    $p->ModuleInstance('{+}/,/');
+    $p->ParameterValueAssignmentArgs('{?}');
+    $p->ModuleInstanceArgs('{+}/,/');
     $p->ValueIs(';');
   }
 );
@@ -487,7 +707,7 @@ Register
     my $p = $_[0];
     $p->ValueIs('#');
     $p->ValueIs('(');
-    $p->PortList('{?}');
+    $p->PortListArgs('{?}');
     $p->ValueIs(')');
 
   }
@@ -501,7 +721,7 @@ Register
     my $p = $_[0];
     $p->TypeIs('Identifier');
     $p->ValueIs('(');
-    $p->PortList('{?}');
+    $p->PortListArgs('{?}');
     $p->ValueIs(')');
 
   }
